----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Carl Mitchell
-- 
-- Create Date: 05/28/2017 11:55:13 AM
-- Design Name: 
-- Module Name: main - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity main is
    Port ( clk : in STD_LOGIC;
           vauxp6 : in STD_LOGIC;
           vauxn6 : in STD_LOGIC;
           vauxp7  : in std_logic;
           vauxn7  : in std_logic;
           vauxp14 : in std_logic;
           vauxn14 : in std_logic;
           vauxp15 : in std_logic;
           vauxn15 : in std_logic;

           txd : out STD_LOGIC;     -- UART transmit.

           uart_led : out std_logic;
           adc_led : out std_logic);
end main;

architecture Behavioral of main is
    signal rst      : std_logic;
    signal reset_sr : std_logic_vector(3 downto 0) := (others => '1');
    component xadc_wiz_1 is
       port
       (
        daddr_in        : in  STD_LOGIC_VECTOR(6 downto 0);      -- Address bus for the dynamic reconfiguration port
        den_in          : in  STD_LOGIC;                         -- Enable Signal for the dynamic reconfiguration port
        di_in           : in  STD_LOGIC_VECTOR(15 downto 0);     -- Input data bus for the dynamic reconfiguration port
        dwe_in          : in  STD_LOGIC;                         -- Write Enable for the dynamic reconfiguration port
        do_out          : out  STD_LOGIC_VECTOR(15 downto 0);    -- Output data bus for dynamic reconfiguration port
        drdy_out        : out  STD_LOGIC;                        -- Data ready signal for the dynamic reconfiguration port
        dclk_in         : in  STD_LOGIC;                         -- Clock input for the dynamic reconfiguration port
        reset_in        : in  STD_LOGIC;                         -- Reset signal for the System Monitor control logic
        vauxp6          : in  STD_LOGIC;                         -- Auxiliary Channel 2
        vauxn6          : in  STD_LOGIC;
        vauxp7          : in  STD_LOGIC;                         -- Auxiliary Channel 3
        vauxn7          : in  STD_LOGIC;
        vauxp14         : in  STD_LOGIC;                         -- Auxiliary Channel 10
        vauxn14         : in  STD_LOGIC;
        vauxp15         : in  STD_LOGIC;                         -- Auxiliary Channel 11
        vauxn15         : in  STD_LOGIC;
        busy_out        : out  STD_LOGIC;                        -- ADC Busy signal
        channel_out     : out  STD_LOGIC_VECTOR(4 downto 0);     -- Channel Selection Outputs
        eoc_out         : out  STD_LOGIC;                        -- End of Conversion Signal
        eos_out         : out  STD_LOGIC;                        -- End of Sequence Signal
        alarm_out       : out STD_LOGIC;                         -- OR'ed output of all the Alarms
        vp_in           : in  STD_LOGIC;                         -- Dedicated Analog Input Pair
        vn_in           : in  STD_LOGIC
    );
    end component;
    signal analogData : std_logic_vector(15 downto 0);
    --signal eoc : std_logic;
    --signal eos : std_logic;
    signal drdy : std_logic;
    constant daddr : std_logic_vector(6 downto 0) := (others => '0'); -- temp --  "0010011"; -- hard selecting channel 3, IE JXADC pin 1 & 7 (rightmost, looking into the connector)
                                                                -- will sequence them later.
    signal enable : std_logic;
    signal channel_out : std_logic_vector(4 downto 0); -- channel selection
    --signal alarm_out : std_logic;
    
    component minimal_uart_tx is
        Port ( clk      : in STD_LOGIC;
               bit_tick : in STD_LOGIC;
               data     : in STD_LOGIC_VECTOR(15 downto 0);
               data_en  : in STD_LOGIC;
               tx       : out STD_LOGIC;
               busy     : out STD_LOGIC);
    end component;
    signal count : unsigned(12 downto 0) := (others => '0');    -- Used to frequency divide the core clock to the transmit clock frequency.
    signal bit_tick : std_logic := '0';                           -- Transmit clock tick.
    constant clk_rate  : unsigned(31 downto 0) := to_unsigned(100000000,32);   -- 100000000, for 100 MHz clock.
    constant baud_rate : unsigned(31 downto 0) := to_unsigned(115200,32);         -- 9600 baud

begin
    
    uart_tx : minimal_uart_tx port map (
        clk      => clk,
        data     => analogData,
        data_en  => drdy,
        bit_tick => bit_tick,
        tx       => txd,
        busy     => uart_led);
        
    adc : xadc_wiz_1 port map (
        daddr_in => daddr,
        den_in => bit_tick, -- was '1'
        di_in => (others => '0'),
        dwe_in => '0',
        do_out => analogData,
        drdy_out => drdy,
        dclk_in => clk,
        reset_in => rst,
        -- JXADC pins
        -- physical numbering and signals
        -- VCC GND 11  2   10  3
        -- 6   5   4   3   2   1 -- Positive
        -- 12  11  10  9   8   7 -- Negative
        vauxp6  => vauxp6, 
        vauxn6  => vauxn6,
        vauxp7  => vauxp7,
        vauxn7  => vauxn7,
        vauxp14 => vauxp14,
        vauxn14 => vauxn14,
        vauxp15 => vauxp15,
        vauxn15 => vauxn15,
        busy_out => adc_led,
        channel_out => channel_out,
        eoc_out => enable,
        eos_out => open,
        alarm_out => open,
        vp_in => '0',
        vn_in => '0'
        );
    
    --enable <= '1';
     rst <= reset_sr(0);
bit_tick_proc:  process(clk)
    begin
        if rising_edge(clk) then
            reset_sr <= '0' & reset_sr(reset_sr'high downto 1);
            if count = clk_rate/baud_rate-1 then
                count <= (others => '0');
                bit_tick <= '1';

            else
                count <= count + 1;
                bit_tick <= '0';
            end if;
        end if;
    end process;

end Behavioral;
