----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Mike Field <hamster@snap.net.nz>, with edits by Carl Mitchell
-- 
-- Create Date: 05/28/2017 04:46:53 PM
-- Design Name: 
-- Module Name: minimal_uart_tx - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;


entity minimal_uart_tx is
    Port ( clk      : in STD_LOGIC;
           bit_tick : in STD_LOGIC;
           data     : in STD_LOGIC_VECTOR(15 downto 0);
           data_en  : in STD_LOGIC;
           tx       : out STD_LOGIC := '0';
           busy     : out STD_LOGIC := '0');
end minimal_uart_tx;

architecture Behavioral of minimal_uart_tx is
    signal shift_register : STD_LOGIC_VECTOR(59 downto 0);  -- Shift register to store the actual data to send
    signal txing          : STD_LOGIC;                     -- flag, are we transmitting data?
    
    signal char0          : STD_LOGIC_VECTOR(7 downto 0);
    signal char1          : STD_LOGIC_VECTOR(7 downto 0);
    signal char2          : STD_LOGIC_VECTOR(7 downto 0);
    signal char3          : STD_LOGIC_VECTOR(7 downto 0);
    
    type t_lookup is array(0 to 15) of std_logic_vector(7 downto 0);
    constant lookup : t_lookup := (
                  x"30", x"31", x"32", x"33",
                  x"34", x"35", x"36", x"37",
                  x"38", x"39", x"41", x"42",
                  x"43", x"44", x"45", x"45");
begin
    busy <= txing;
    with shift_register(9 downto 0) select
        txing <= '0' when "0000000000", '1' when others;
    
    -- ASCII Lookup
    char0 <= lookup(to_integer(unsigned(data( 3 downto  0))));
    char1 <= lookup(to_integer(unsigned(data( 7 downto  4))));
    char2 <= lookup(to_integer(unsigned(data(11 downto  8))));
    char3 <= lookup(to_integer(unsigned(data(15 downto 12))));
    
    clk_process : process(clk)
    begin
        if rising_edge(clk) then
            if txing = '0' and data_en = '1' then   -- If not transmitting, and there's data incoming, 
                                                    -- then load data with start and stop bits into the shift register
                shift_register <= '1' & "00001010" & '0'
                                & '1' & "00001101" & '0'
                                & '1' & char0 & '0' 
                                & '1' & char1 & '0' 
                                & '1' & char2 & '0' 
                                & '1' & char3 & '0'; -- Start bit, data, stop bit
            end if;
            
            if bit_tick = '1' and txing = '1' then                   -- Transmit clock is high and we have data to transmit
                tx <= shift_register(0);                             -- Shift out a bit of data
                shift_register <= '0' & shift_register(59 downto 1); -- and shift in a zero
            end if;
        end if;
    end process;

end Behavioral;
